Package: eisaR
Title: Exon-Intron Split Analysis (EISA) in R
Version: 1.25.0
Authors@R: c(person("Michael", "Stadler", email = "michael.stadler@fmi.ch", role = c("aut", "cre")),
             person("Dimos", "Gaidatzis", email = "dimosthenis.gaidatzis@fmi.ch", role = "aut"),
             person("Lukas", "Burger", email = "lukas.burger@fmi.ch", role = "aut"),
             person("Charlotte", "Soneson", email = "charlotte.soneson@fmi.ch", role = "aut"))
Description: Exon-intron split analysis (EISA) uses ordinary RNA-seq data to measure
    changes in mature RNA and pre-mRNA reads across different experimental conditions
    to quantify transcriptional and post-transcriptional regulation of gene expression.
    For details see Gaidatzis et al., Nat Biotechnol 2015. doi: 10.1038/nbt.3269.
    eisaR implements the major steps of EISA in R. 
Depends: R (>= 4.1)
License: GPL-3
biocViews: Transcription, GeneExpression, GeneRegulation,
        FunctionalGenomics, Transcriptomics, Regression, RNASeq
Encoding: UTF-8
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Suggests: knitr, rmarkdown, testthat, BiocStyle, QuasR, Rbowtie,
        Rhisat2, Biostrings, BSgenome, BSgenome.Hsapiens.UCSC.hg38,
        ensembldb, AnnotationDbi, GenomicFeatures, txdbmaker,
        rtracklayer, withr
VignetteBuilder: knitr
Imports: graphics, stats, GenomicRanges, S4Vectors, IRanges, limma,
        edgeR (>= 4.0), methods, SummarizedExperiment, BiocGenerics,
        utils
URL: https://github.com/fmicompbio/eisaR
BugReports: https://github.com/fmicompbio/eisaR/issues
git_url: https://git.bioconductor.org/packages/eisaR
git_branch: devel
git_last_commit: 939131c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:29:33 UTC; biocbuild
Author: Michael Stadler [aut, cre],
  Dimos Gaidatzis [aut],
  Lukas Burger [aut],
  Charlotte Soneson [aut]
Maintainer: Michael Stadler <michael.stadler@fmi.ch>
