# Generated by roxygen2: do not edit by hand

export(exportToGtf)
export(getFeatureRanges)
export(getRegionsFromTxDb)
export(getTx2Gene)
export(plotEISA)
export(runEISA)
import(GenomicRanges)
import(edgeR)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,relist)
importFrom(BiocGenerics,setdiff)
importFrom(BiocGenerics,sort)
importFrom(BiocGenerics,start)
importFrom(BiocGenerics,subset)
importFrom(BiocGenerics,unlist)
importFrom(GenomicRanges,GRangesList)
importFrom(GenomicRanges,reduce)
importFrom(IRanges,IRanges)
importFrom(S4Vectors,elementNROWS)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,runLength)
importFrom(S4Vectors,split)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(graphics,legend)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(limma,nonEstimable)
importFrom(methods,is)
importFrom(stats,model.matrix)
importFrom(utils,write.table)
